@@ -18,14 +18,19 @@
 import os, sys
 import glob
 import locale
+import logging
 import gc
+import genshi
+
+from config import LazygalConfig, LazygalWebgalConfig,\
+                   USER_CONFIG_PATH, LazygalConfigDeprecated
+from sourcetree import SOURCEDIR_CONFIGFILE
 
 import make
-import sourcetree, tpl, newsize, metadata, mediautils
+import pathutils
+import sourcetree, tpl, newsize, metadata
 import genpage, genmedia, genfile
 
-from sourcetree import SOURCEDIR_CONFIGFILE
-
 
 from lazygal import INSTALL_MODE, INSTALL_PREFIX
 if INSTALL_MODE == 'source':
@@ -52,37 +57,37 @@ class SubgalSort(make.MakeTask):
         make.MakeTask.__init__(self)
         self.set_dep_only()
         self.webgal_dir = webgal_dir
-        self.album = self.webgal_dir.album
 
     def build(self):
-        self.album.log(_("  SORTING pics and subdirs"), 'info')
+        logging.info(_("  SORTING pics and subdirs"))
 
-        if self.album.subgal_sort_by[0] == 'exif':
+        if self.webgal_dir.subgal_sort_by[0] == 'exif':
             subgal_sorter = lambda x, y:\
                                 x.source_dir.compare_latest_exif(y.source_dir)
-        elif self.album.subgal_sort_by[0] == 'mtime':
+        elif self.webgal_dir.subgal_sort_by[0] == 'mtime':
             subgal_sorter = lambda x, y:\
                                 x.source_dir.compare_mtime(y.source_dir)
-        elif self.album.subgal_sort_by[0] == 'dirname'\
-        or self.album.subgal_sort_by[0] == 'filename': # Backward compatibility
+        elif self.webgal_dir.subgal_sort_by[0] == 'dirname'\
+        or self.webgal_dir.subgal_sort_by[0] == 'filename': # Backward compatibility
             subgal_sorter = lambda x, y:\
                                 x.source_dir.compare_filename(y.source_dir)
         else:
             raise ValueError(_("Unknown sorting criterion '%s'")\
-                             % self.album.subgal_sort_by[0])
+                             % self.webgal_dir.subgal_sort_by[0])
         self.webgal_dir.subgals.sort(subgal_sorter,
-                                     reverse=self.album.subgal_sort_by[1])
+                                     reverse=self.webgal_dir.subgal_sort_by[1])
 
-        if self.album.pic_sort_by[0] == 'exif':
+        if self.webgal_dir.pic_sort_by[0] == 'exif':
             sorter = lambda x, y: x.media.compare_to_sort(y.media)
-        elif self.album.pic_sort_by[0] == 'mtime':
+        elif self.webgal_dir.pic_sort_by[0] == 'mtime':
             sorter = lambda x, y: x.media.compare_mtime(y.media)
-        elif self.album.pic_sort_by[0] == 'filename':
+        elif self.webgal_dir.pic_sort_by[0] == 'filename':
             sorter = lambda x, y: x.media.compare_filename(y.media)
         else:
             raise ValueError(_("Unknown sorting criterion '%s'")\
-                             % self.album.pic_sort_by[0])
-        self.webgal_dir.medias.sort(sorter, reverse=self.album.pic_sort_by[1])
+                             % self.webgal_dir.pic_sort_by[0])
+        self.webgal_dir.medias.sort(sorter,
+                                    reverse=self.webgal_dir.pic_sort_by[1])
 
         # chain medias
         previous = None
@@ -101,7 +106,6 @@ class SubgalBreak(make.MakeTask):
     def __init__(self, webgal_dir):
         make.MakeTask.__init__(self)
         self.webgal_dir = webgal_dir
-        self.album = self.webgal_dir.album
 
         self.__last_page_number = -1
 
@@ -113,9 +117,9 @@ class SubgalBreak(make.MakeTask):
         return self.__last_page_number + 1
 
     def build(self):
-        self.album.log(_("  BREAKING web gallery into multiple pages"), 'info')
+        logging.info(_("  BREAKING web gallery into multiple pages"))
 
-        if self.album.thumbs_per_page == 0:
+        if self.webgal_dir.thumbs_per_page == 0:
             self.__fill_no_pagination()
         else:
             if self.webgal_dir.flatten_below():
@@ -148,7 +152,7 @@ class SubgalBreak(make.MakeTask):
                 subgal = subgals_it.next()
                 how_many_medias += subgal.source_dir.get_media_count()
                 galleries.append((subgal, subgal.medias))
-                if how_many_medias > self.webgal_dir.album.thumbs_per_page:
+                if how_many_medias > self.webgal_dir.thumbs_per_page:
                     self.webgal_dir.add_index_page(subgals, galleries)
                     galleries = []
                     how_many_medias = 0
@@ -158,10 +162,10 @@ class SubgalBreak(make.MakeTask):
 
     def __fill_real_pagination(self):
         how_many_pages = len(self.webgal_dir.medias)\
-                          // self.webgal_dir.album.thumbs_per_page + 1
+                          // self.webgal_dir.thumbs_per_page + 1
         for page_number in range(0, how_many_pages):
-            step = page_number * self.webgal_dir.album.thumbs_per_page
-            end_index = step + self.webgal_dir.album.thumbs_per_page
+            step = page_number * self.webgal_dir.thumbs_per_page
+            end_index = step + self.webgal_dir.thumbs_per_page
             shown_medias = self.webgal_dir.medias[step:end_index]
             galleries = [(self.webgal_dir, shown_medias)]
 
@@ -176,10 +180,9 @@ class SubgalBreak(make.MakeTask):
 
 class WebalbumMediaTask(make.GroupTask):
 
-    def __init__(self, webgal, media, album):
+    def __init__(self, webgal, media):
         super(WebalbumMediaTask, self).__init__()
 
-        self.album = album
         self.webgal = webgal
         self.media = media
 
@@ -190,14 +193,14 @@ class WebalbumMediaTask(make.GroupTask):
         self.resized = {}
         self.browse_pages = {}
 
-        for size_name in self.album.browse_size_strings.keys():
-            if self.album.browse_size_strings[size_name] == '0x0':
+        for size_name in self.webgal.browse_sizes:
+            if self.webgal.newsizers[size_name] == 'original':
                 self.resized[size_name] = self.get_original()
             else:
                 self.resized[size_name] = self.get_resized(size_name)
             self.add_dependency(self.resized[size_name])
 
-            if self.album.original and not self.album.orig_base:
+            if self.webgal.original and not self.webgal.orig_base:
                 self.add_dependency(self.get_original())
 
             self.browse_pages[size_name] = self.get_browse_page(size_name)
@@ -215,7 +218,7 @@ class WebalbumMediaTask(make.GroupTask):
                 if media.thumb: bpage.add_dependency(media.thumb)
 
     def get_original_or_symlink(self):
-        if not self.album.orig_symlink:
+        if not self.webgal.orig_symlink:
             return genfile.MediaOriginal(self.webgal, self.media)
         else:
             return genfile.SymlinkMediaOriginal(self.webgal, self.media)
@@ -231,15 +234,15 @@ class WebalbumImageTask(WebalbumMediaTask):
     This task builds all items related to one picture.
     """
 
-    def __init__(self, webgal, image, album):
-        super(WebalbumImageTask, self).__init__(webgal, image, album)
+    def __init__(self, webgal, image):
+        super(WebalbumImageTask, self).__init__(webgal, image)
 
         self.thumb = genmedia.ImageOtherSize(self.webgal, self.media,
                                              genmedia.THUMB_SIZE_NAME)
         self.add_dependency(self.thumb)
 
     def get_resized(self, size_name):
-        if self.album.browse_size_strings[size_name] == '0x0':
+        if self.webgal.newsizers[size_name] == 'original':
             return self.get_original_or_symlink()
         else:
             return genmedia.ImageOtherSize(self.webgal, self.media, size_name)
@@ -253,12 +256,13 @@ class WebalbumVideoTask(WebalbumMediaTask):
     This task builds all items related to one video.
     """
 
-    def __init__(self, webgal, video, album):
+    def __init__(self, webgal, video):
         self.webvideo = None
 
-        super(WebalbumVideoTask, self).__init__(webgal, video, album)
+        super(WebalbumVideoTask, self).__init__(webgal, video)
 
-        self.thumb = None # none yet
+        self.thumb = genmedia.VideoThumb(self.webgal, self.media,
+                                         genmedia.THUMB_SIZE_NAME)
 
         self.add_dependency(self.webvideo)
 
@@ -267,7 +271,10 @@ class WebalbumVideoTask(WebalbumMediaTask):
 
     def get_resized(self, size_name):
         if not self.webvideo:
-            self.webvideo = genmedia.WebVideo(self.webgal, self.media)
+            if self.media.extension == '.webm':
+                self.webvideo = self.get_original()
+            else:
+                self.webvideo = genmedia.WebVideo(self.webgal, self.media)
         return self.webvideo
 
 
@@ -276,7 +283,7 @@ class WebalbumDir(make.FileMakeObject):
     This is a built web gallery with its files, thumbs and reduced pics.
     """
 
-    def __init__(self, dir, subgals, album, album_dest_dir, clean_dest=False):
+    def __init__(self, dir, subgals, album, album_dest_dir):
         self.source_dir = dir
         self.path = os.path.join(album_dest_dir, self.source_dir.strip_root())
         if self.path.endswith('/'): self.path = os.path.dirname(self.path)
@@ -286,16 +293,18 @@ class WebalbumDir(make.FileMakeObject):
         self.add_dependency(self.source_dir)
         self.subgals = subgals
         self.album = album
+        self.feed = None
 
         self.flattening_dir = None
 
-        self.clean_dest = clean_dest
+        self.config = LazygalWebgalConfig(self.album.config)
+        self.__configure()
 
         # Create the directory if it does not exist
         if not os.path.isdir(self.path):
-            self.album.log(_("  MKDIR %%WEBALBUMROOT%%/%s")\
-                           % self.source_dir.strip_root(), 'info')
-            self.album.log("(%s)" % self.path)
+            logging.info(_("  MKDIR %%WEBALBUMROOT%%/%s")\
+                         % self.source_dir.strip_root())
+            logging.debug("(%s)" % self.path)
             os.makedirs(self.path, mode = 0755)
             # Directory did not exist, mark it as so
             self.stamp_delete()
@@ -307,9 +316,9 @@ class WebalbumDir(make.FileMakeObject):
             self.sort_task.add_dependency(media)
 
             if media.type == 'image':
-                media_task = WebalbumImageTask(self, media, self.album)
+                media_task = WebalbumImageTask(self, media)
             elif media.type == 'video':
-                media_task = WebalbumVideoTask(self, media, self.album)
+                media_task = WebalbumVideoTask(self, media)
             else:
                 raise NotImplementedError("Unknown media type '%s'"\
                                           % media.type)
@@ -318,35 +327,171 @@ class WebalbumDir(make.FileMakeObject):
 
         self.dirzip = None
 
+        self.index_pages = []
+
         if not self.should_be_flattened():
             self.break_task = SubgalBreak(self)
-            # This task is special because it populates dependencies. This is
-            # why it needs to be built before a build check.
-            self.break_task.make()
 
-            if self.album.thumbs_per_page > 0:
+            if self.thumbs_per_page > 0:
                 # FIXME: If pagination is 'on', galleries need to be sorted
                 # before being broken on multiple pages, and thus this slows
                 # down a lot the checking of a directory's need to be built.
                 self.break_task.add_dependency(self.sort_task)
 
+            # This task is special because it populates dependencies. This is
+            # why it needs to be built before a build check.
+            self.break_task.make()
+
             self.webgal_pic = genmedia.WebalbumPicture(self)
             self.add_dependency(self.webgal_pic)
         else:
             self.break_task = None
 
+    def __parse_browse_sizes(self, sizes_string):
+        for single_def in sizes_string.split(','):
+            name, string_size = single_def.split('=')
+            if name == '':
+                raise ValueError(_("Sizes is a comma-separated list of size names and specs:\n\t e.g. \"small=640x480,medium=1024x768\"."))
+            if name == genmedia.THUMB_SIZE_NAME:
+                raise ValueError(_("Size name '%s' is reserved for internal processing.") % genmedia.THUMB_SIZE_NAME)
+            self.__parse_size(name, string_size)
+            self.browse_sizes.append(name)
+
+    def __parse_size(self, size_name, size_string):
+        if size_string == '0x0':
+            self.newsizers[size_name] = 'original'
+        else:
+            try:
+                self.newsizers[size_name] = newsize.get_newsizer(size_string)
+            except newsize.NewsizeStringParseError:
+                raise ValueError(_("'%s' for size '%s' does not describe a known size syntax.") % (size_string, size_name, ))
+
+    def __parse_sort(self, sort_string):
+        try:
+            sort_method, reverse = sort_string.split(':')
+        except ValueError:
+            sort_method = sort_string
+            reverse = False
+        if reverse == 'reverse':
+            return sort_method, True
+        else:
+            return sort_method, False
+
+    def __load_tpl_vars(self):
+        # Load tpl vars from config
+        tpl_vars = {}
+        if self.config.has_section('template-vars'):
+            tpl_vars = {}
+            for option in self.config.options('template-vars'):
+                value = self.config.get('template-vars', option)
+                value = value.decode(locale.getpreferredencoding())
+                tpl_vars[option] = genshi.core.Markup(value)
+
+        return tpl_vars
+
+    def __configure(self):
+        config_dirs = self.source_dir.parent_paths()[:-1] # strip root dir
+        config_dirs.reverse() # from root to deepest
+        config_files = map(lambda d: os.path.join(d, SOURCEDIR_CONFIGFILE),
+                           config_dirs)
+        logging.debug(_("  Trying loading gallery configs: %s")
+                      % ', '.join(map(self.source_dir.strip_root,
+                                          config_files)))
+        self.config.read(config_files)
+
+        self.browse_sizes = []
+        self.newsizers = {}
+        self.__parse_browse_sizes(self.config.get('webgal', 'image-size'))
+        self.__parse_size(genmedia.THUMB_SIZE_NAME,
+                          self.config.get('webgal', 'thumbnail-size'))
+        self.default_size_name = self.browse_sizes[0]
+
+        self.tpl_vars = self.__load_tpl_vars()
+        styles = self.album.get_avail_styles(self.album.theme,
+                                     self.config.get('webgal', 'default-style'))
+        self.tpl_vars.update({'styles' : styles})
+
+        self.set_original(self.config.getboolean('webgal', 'original'),
+                          self.config.getstr('webgal', 'original-baseurl'),
+                          self.config.getboolean('webgal', 'original-symlink'))
+
+        self.thumbs_per_page = self.config.getint('webgal', 'thumbs-per-page')
+
+        if self.config.getboolean('webgal', 'dirzip')\
+        and self.source_dir.get_media_count() > 1:
+            self.dirzip = genfile.WebalbumArchive(dir)
+            self.add_dependency(self.dirzip)
+        else:
+            self.dirzip = None
+
+        self.quality = self.config.getint('webgal', 'jpeg-quality')
+        self.save_options = {}
+        if self.config.getboolean('webgal', 'jpeg-optimize'):
+            self.save_options['optimize'] = True
+        if self.config.getboolean('webgal', 'jpeg-progressive'):
+            self.save_options['progressive'] = True
+
+        self.pic_sort_by = self.__parse_sort(self.config.get('webgal', 'sort-medias'))
+        self.subgal_sort_by = self.__parse_sort(self.config.get('webgal', 'sort-subgals'))
+
+        self.webalbumpic_bg = self.config.get('webgal', 'webalbumpic-bg')
+        try:
+            self.webalbumpic_size = map(int, self.config.get('webgal', 'webalbumpic-size').split('x'))
+            if len(self.webalbumpic_size) != 2:
+                raise ValueError
+        except ValueError:
+            logging.error(_('Bad syntax for webalbumpic-size.'))
+            sys.exit(1)
+
+    def set_original(self, original=False, orig_base=None, orig_symlink=False):
+        self.original = original or orig_symlink
+        self.orig_symlink = orig_symlink
+        if self.original and orig_base and not orig_symlink:
+            self.orig_base = orig_base
+        else:
+            self.orig_base = None
+
+    def get_webalbumpic_filename(self):
+        if self.webalbumpic_bg == 'transparent':
+            ext = '.png' # JPEG does not have an alpha channel
+        else:
+            ext = '.jpg'
+        return genmedia.WebalbumPicture.BASEFILENAME + ext
+
+    def _add_size_qualifier(self, path, size_name, force_extension=None):
+        filename, extension = os.path.splitext(path)
+        if force_extension is not None:
+            extension = force_extension
+
+        if size_name == self.default_size_name and extension == '.html':
+            # Do not append default size name to HTML page filename
+            return path
+        elif size_name in self.browse_sizes\
+        and self.newsizers[size_name] == 'original'\
+        and extension != '.html':
+            # Do not append size_name to unresized images.
+            return path
+        else:
+            return "%s_%s%s" % (filename, size_name, extension)
+
     def add_index_page(self, subgals, galleries):
         page_number = self.break_task.next_page_number()
-        for size_name in self.album.browse_size_strings.keys():
+        pages = []
+        for size_name in self.browse_sizes:
             page = genpage.WebalbumIndexPage(self, size_name, page_number,
                                              subgals, galleries)
             self.add_dependency(page)
+            pages.append(page)
+        self.index_pages.append(pages)
 
     def register_output(self, output):
         # We only care about output in the current directory
         if os.path.dirname(output) == self.path:
             super(WebalbumDir, self).register_output(output)
 
+    def register_feed(self, feed):
+        self.feed = feed
+
     def get_subgal_count(self):
         if self.flatten_below():
             return 0
@@ -380,7 +525,7 @@ class WebalbumDir(make.FileMakeObject):
         else:
             return False
 
-    def rel_path(self, target_srcdir_path):
+    def rel_path_to_src(self, target_srcdir_path):
         '''
         Returns the relative path to go from this directory to
         target_srcdir_path.
@@ -388,6 +533,13 @@ class WebalbumDir(make.FileMakeObject):
         return self.source_dir.rel_path(self.source_dir.path,
                                         target_srcdir_path)
 
+    def rel_path(self, path):
+        '''
+        Returns the relative path to go from this directory to the path supplied
+        as argument.
+        '''
+        return pathutils.relative_path(self.path, path)
+
     def flattening_srcpath(self, srcdir_path):
         '''
         Returns the source path in which srcdir_path should flattened, that is
@@ -426,13 +578,7 @@ class WebalbumDir(make.FileMakeObject):
 
     def build(self):
         for dest_file in self.list_foreign_files():
-            text = ''
-            if self.clean_dest and not os.path.isdir(dest_file):
-                os.unlink(dest_file)
-                text = ""
-            else:
-                text = _("you should")
-            self.album.log(_("  %s RM %s") % (text, dest_file), 'info')
+            self.album.cleanup(dest_file)
 
     def make(self, force=False):
         needed_build = self.needs_build()
@@ -447,89 +593,81 @@ class WebalbumDir(make.FileMakeObject):
         if needed_build: os.utime(self.path, None)
 
 
-class SharedFiles(make.FileSimpleDependency):
+class SharedFiles(make.FileMakeObject):
 
     def __init__(self, album, dest_dir):
         self.path = os.path.join(dest_dir, DEST_SHARED_DIRECTORY_NAME)
+        self.album = album
 
         # Create the shared files directory if it does not exist
         if not os.path.isdir(self.path):
-            album.log(_("MKDIR %SHAREDDIR%"), 'info')
-            album.log("(%s)" % self.path)
+            logging.info(_("MKDIR %SHAREDDIR%"))
+            logging.debug("(%s)" % self.path)
             os.makedirs(self.path, mode = 0755)
 
         super(SharedFiles, self).__init__(self.path)
 
+        self.expected_shared_files = []
         for shared_file in glob.glob(\
-          os.path.join(album.tpl_dir, THEME_SHARED_FILE_PREFIX + '*')):
+          os.path.join(self.album.tpl_dir, THEME_SHARED_FILE_PREFIX + '*')):
             shared_file_name = os.path.basename(shared_file).\
                                      replace(THEME_SHARED_FILE_PREFIX, '')
             shared_file_dest = os.path.join(self.path,
                                             shared_file_name)
 
-            if album.tpl_loader.is_known_template_type(shared_file):
-                self.add_dependency(genpage.SharedFileTemplate(album,
-                                                        shared_file,
-                                                        shared_file_dest))
+            if self.album.tpl_loader.is_known_template_type(shared_file):
+                sf = genpage.SharedFileTemplate(album, shared_file,
+                                                shared_file_dest)
+                self.expected_shared_files.append(sf.path)
             else:
-                self.add_dependency(genfile.SharedFileCopy(album,
-                                                           shared_file,
-                                                           shared_file_dest))
+                sf = genfile.SharedFileCopy(shared_file, shared_file_dest)
+                self.expected_shared_files.append(shared_file_dest)
+
+            self.add_dependency(sf)
+
+    def build(self):
+        # Cleanup themes files which are not in themes anymore.
+        for present_file in os.listdir(self.path):
+            file_path = os.path.join(self.path, present_file)
+            if file_path not in self.expected_shared_files:
+                self.album.cleanup(file_path)
 
 
 class Album:
 
-    def __init__(self, source_dir,
-                 thumb_size_string='150x113', browse_size_strings=None,
-                 optimize=False, progressive=False, quality=85,
-                 dir_flattening_depth=False, thumbs_per_page=0,
-                 dirzip=False,
-                 pic_sort_by=('exif', False),
-                 subgal_sort_by=('filename', False)):
-        self.set_logging()
-
+    def __init__(self, source_dir, config=None):
         self.source_dir = os.path.abspath(source_dir)
-        self.source_dir = self.source_dir.decode(sys.getfilesystemencoding())
 
-        self.thumb_size_string = thumb_size_string
-        if browse_size_strings is not None:
-            self.browse_size_strings = dict(browse_size_strings)
-            self.default_size_name = browse_size_strings[0][0]
-        else:
-            self.browse_size_strings = {'small': '800x600',
-                                        'medium': '1024x768'}
-            self.default_size_name = 'small'
+        self.config = LazygalConfig()
 
-        self.newsizers = {}
-        for size_name, size_string in self.browse_size_strings.items():
-            self.newsizers[size_name] = newsize.get_newsizer(size_string)
-        self.newsizers[genmedia.THUMB_SIZE_NAME] = newsize.get_newsizer(self.thumb_size_string)
+        logging.info(_("Trying loading user config %s") % USER_CONFIG_PATH)
+        self.config.read(USER_CONFIG_PATH)
 
-        self.quality = quality
+        sourcedir_configfile = os.path.join(source_dir, SOURCEDIR_CONFIGFILE)
+        if os.path.isfile(sourcedir_configfile):
+            logging.info(_("Loading root config %s") % sourcedir_configfile)
+            try:
+                self.config.read(sourcedir_configfile)
+            except LazygalConfigDeprecated:
+                logging.error(_("'%s' uses a deprecated syntax: please refer to lazygal.conf(5) manual page.") % sourcedir_configfile)
+                sys.exit(1)
+        if config is not None: # Supplied config
+            self.config.load(config)
+
+        if self.config.getboolean('runtime', 'quiet'):
+            logging.getLogger().setLevel(logging.ERROR)
+        if self.config.getboolean('runtime', 'debug'):
+            logging.getLogger().setLevel(logging.DEBUG)
+
+        self.clean_dest = self.config.getboolean('global', 'clean-destination')
 
         self.tpl_loader = None
-        self.tpl_vars = {}
-        self.original = False
-        self.orig_base = None
-        self.orig_symlink = False
-        self.thumbs_per_page = thumbs_per_page
-        self.dir_flattening_depth = dir_flattening_depth
-        self.dirzip = dirzip
-        self.save_options = {}
-        if optimize:
-            self.save_options['optimize'] = True
-        if progressive:
-            self.save_options['progressive'] = True
 
-        self.pic_sort_by = pic_sort_by
-        self.subgal_sort_by = subgal_sort_by
+        self.set_theme(self.config.get('global', 'theme'))
 
-        self.transcoder = None
-        self.videothumb = None
+        self.dir_flattening_depth = self.config.getint('global', 'dir-flattening-depth')
 
-        self.set_webalbumpic()
-
-    def set_theme(self, theme=tpl.DEFAULT_TEMPLATE, default_style=None):
+    def set_theme(self, theme=tpl.DEFAULT_TEMPLATE):
         self.theme = theme
 
         # First try user directory
@@ -544,12 +682,8 @@ class Album:
                                                       tpl.DEFAULT_TEMPLATE),
                                          self.tpl_dir)
 
-        styles = self.get_avail_styles(self.theme, default_style)
-        self.tpl_vars.update({'styles' : styles})
-        self.set_tpl_vars()
-
         # Load styles templates
-        for style in styles:
+        for style in self.get_avail_styles(theme):
             style_filename = style['filename']
             try:
                 self.tpl_loader.load(style_filename)
@@ -557,21 +691,7 @@ class Album:
                 # Not a known emplate ext, ignore
                 pass
 
-    def get_transcoder(self):
-        if self.transcoder is None:
-            if mediautils.HAVE_GST:
-                self.transcoder = mediautils.OggTheoraTranscoder()
-            else:
-                self.transcoder = False
-        return self.transcoder
-
-    def set_tpl_vars(self, tpl_vars=None):
-        if tpl_vars is not None:
-            self.tpl_vars.update(tpl_vars)
-        if self.tpl_loader is not None:
-            self.tpl_loader.set_common_values(self.tpl_vars)
-
-    def get_avail_styles(self, theme, default_style):
+    def get_avail_styles(self, theme, default_style=None):
         style_files_mask = os.path.join(self.tpl_dir,
                                         THEME_SHARED_FILE_PREFIX + '*' + 'css')
         styles = []
@@ -580,76 +700,29 @@ class Album:
             tpl_filename = os.path.basename(style_tpl_file).split('.')[0]
             style['filename'] = tpl_filename[len(THEME_SHARED_FILE_PREFIX):]
             style['name'] = self._str_humanize(style['filename'])
-            if style['filename'] == default_style:
-                style['alternate'] = False
-            else:
-                style['alternate'] = True
+            if default_style is not None:
+                if style['filename'] == default_style:
+                    style['rel'] = 'stylesheet'
+                else:
+                    style['rel'] = 'alternate stylesheet'
             styles.append(style)
         return styles
 
-    def set_original(self, original=False, orig_base=None, orig_symlink=False):
-        self.original = original
-        self.orig_symlink = orig_symlink
-        if self.original and orig_base and not orig_symlink:
-            self.orig_base = orig_base
-        else:
-            self.orig_base = None
-
-    def set_webalbumpic(self, bg='transparent'):
-        self.webalbumpic_bg = bg
-
-    def get_webalbumpic_filename(self):
-        if self.webalbumpic_bg == 'transparent':
-            ext = '.png' # JPEG does not have an alpha channel
-        else:
-            ext = '.jpg'
-        return genmedia.WebalbumPicture.BASEFILENAME + ext
-
-    log_levels = ['debug', 'info', 'error']
-
-    def set_logging(self, level='info', outpipe=sys.stdout,
-                                           errpipe=sys.stderr):
-        self.log_level = level
-        self.log_outpipe = outpipe
-        self.log_errpipe = errpipe
-
-    def log(self, msg, level='debug'):
-        if self.log_levels.index(level)\
-           >= self.log_levels.index(self.log_level):
-            msg = msg.encode(locale.getpreferredencoding())
-            if level == 'error':
-                print >> self.log_errpipe, msg
-                self.log_errpipe.flush()
-            else:
-                print >> self.log_outpipe, msg
-                self.log_outpipe.flush()
-
-    def _add_size_qualifier(self, path, size_name):
-        filename, extension = os.path.splitext(path)
-        if size_name == self.default_size_name and extension == '.html':
-            # Do not append default size name to HTML page filename
-            return path
-        elif size_name in self.browse_size_strings.keys()\
-        and self.browse_size_strings[size_name] == '0x0'\
-        and extension != '.html':
-            # Do not append size_name to unresized images.
-            return path
-        else:
-            return "%s_%s%s" % (filename, size_name, extension)
-
     def _str_humanize(self, text):
         dash_replaced = text.replace('_', ' ')
         return dash_replaced
 
     def is_in_sourcetree(self, path):
-        head = path
-        old_head = None
-        while head != old_head:
-            if head == self.source_dir:
-                return True
-            old_head = head
-            head, tail = os.path.split(head)
-        return False
+        return pathutils.is_subdir_of(self.source_dir, path)
+
+    def cleanup(self, file_path):
+        text = ''
+        if self.clean_dest and not os.path.isdir(file_path):
+            os.unlink(file_path)
+            text = ''
+        else:
+            text = _('you should ')
+        logging.info(_('  %sRM %s') % (text, file_path))
 
     def walk(self, top, walked=None):
         '''
@@ -673,7 +746,7 @@ class Album:
                         for x in self.walk(d_path, walked):
                             yield x
                     else:
-                        self.log(_("Not following symlink '%s' because directory has already been processed.") % d_path, 'error')
+                        logging.warning(_("Not following symlink '%s' because directory has already been processed.") % d_path)
 
             yield root, dirs, files
 
@@ -681,27 +754,31 @@ class Album:
         '''
         Generate default metada files if no exists.
         '''
-        self.log(_("Generating metadata in %s") % self.source_dir)
+        logging.debug(_("Generating metadata in %s") % self.source_dir)
 
         for root, dirnames, filenames in self.walk(self.source_dir):
             filenames.sort() # This is required for the ignored files
                              # checks to be reliable.
             source_dir = sourcetree.Directory(root, [], filenames, self)
-            self.log(_("[Entering %%ALBUMROOT%%/%s]") % source_dir.strip_root(),
-                     'info')
-            self.log("(%s)" % source_dir.path)
+            logging.info(_("[Entering %%ALBUMROOT%%/%s]") % source_dir.strip_root())
+            logging.debug("(%s)" % source_dir.path)
 
             metadata.DefaultMetadata(source_dir, self).make()
 
-    def generate(self, dest_dir, pub_url=None,
-                 check_all_dirs=False, clean_dest=False):
-        dest_dir = dest_dir.decode(sys.getfilesystemencoding())
-        sane_dest_dir = os.path.abspath(dest_dir)
+    def generate(self, dest_dir=None):
+        if dest_dir is None:
+            dest_dir = self.config.getstr('global', 'destdir')
+        else:
+            dest_dir = dest_dir.decode(sys.getfilesystemencoding())
+        sane_dest_dir = os.path.abspath(os.path.expanduser(dest_dir))
+
+        pub_url = self.config.getstr('global', 'puburl')
+        check_all_dirs = self.config.getboolean('runtime', 'check-all-dirs')
 
         if self.is_in_sourcetree(sane_dest_dir):
             raise ValueError(_("Fatal error, web gallery directory is within source tree."))
 
-        self.log(_("Generating to %s") % sane_dest_dir)
+        logging.debug(_("Generating to %s") % sane_dest_dir)
 
         if pub_url:
             feed = genpage.WebalbumFeed(self, sane_dest_dir, pub_url)
@@ -718,27 +795,27 @@ class Album:
                 subdirs = []
                 subgals = []
 
+            checked_dir = sourcetree.File(root, self)
+
+            if checked_dir.should_be_skipped():
+                logging.debug(_("(%s) has been skipped") % checked_dir.path)
+                continue
+            if checked_dir.path == os.path.join(sane_dest_dir,
+                                                DEST_SHARED_DIRECTORY_NAME):
+                logging.error(_("(%s) has been skipped because its name collides with the shared material directory name") % checked_dir.path)
+                continue
+
+            logging.info(_("[Entering %%ALBUMROOT%%/%s]") % checked_dir.strip_root())
+            logging.debug("(%s)" % checked_dir.path)
+
             source_dir = sourcetree.Directory(root, subdirs, filenames, self)
 
-            if source_dir.should_be_skipped():
-                self.log(_("(%s) has been skipped") % source_dir.path)
-                continue
-            if source_dir.path == os.path.join(sane_dest_dir,
-                                               DEST_SHARED_DIRECTORY_NAME):
-                self.log(_("(%s) has been skipped because its name collides with the shared material directory name") % source_dir.path, 'error')
-                continue
-
-            self.log(_("[Entering %%ALBUMROOT%%/%s]") % source_dir.strip_root(),
-                     'info')
-            self.log("(%s)" % source_dir.path)
-
             if source_dir.get_all_medias_count() < 1:
-                self.log(_("(%s) and childs have no known medias, skipped")
-                           % source_dir.path)
+                logging.debug(_("(%s) and childs have no known medias, skipped")
+                              % source_dir.path)
                 continue
 
-            destgal = WebalbumDir(source_dir, subgals, self,
-                                  sane_dest_dir, clean_dest)
+            destgal = WebalbumDir(source_dir, subgals, self, sane_dest_dir)
 
             if not source_dir.is_album_root():
                 container_dirname = os.path.dirname(root)
@@ -757,25 +834,26 @@ class Album:
 
             if feed:
                 feed.push_dir(destgal)
+                destgal.register_feed(feed)
 
             if check_all_dirs:
                 destgal.make()
             elif destgal.needs_build():
                 destgal.make(force=True) # avoid another needs_build() call in make()
             else:
-                self.log(_("  SKIPPED because of mtime, touch source or use --check-all-dirs to override."))
+                logging.debug(_("  SKIPPED because of mtime, touch source or use --check-all-dirs to override."))
 
             # Force some memory cleanups, this is usefull for big albums.
             del destgal
             gc.collect()
 
-            self.log(_("[Leaving  %%ALBUMROOT%%/%s]") % source_dir.strip_root(),
-                     'info')
+            logging.info(_("[Leaving  %%ALBUMROOT%%/%s]") % source_dir.strip_root())
 
         if feed:
             feed.make()
 
-        SharedFiles(self, sane_dest_dir).make()
+        # Force to check for unexpected files
+        SharedFiles(self, sane_dest_dir).make(True)
 
 
 # vim: ts=4 sw=4 expandtab
