@@ -18,12 +18,14 @@
 
 import os
 import sys
+import logging
 import urllib
 import locale
 
 import genshi
 
 import make
+import pathutils
 import genfile, genmedia, feeds
 
 
@@ -50,67 +52,36 @@ class WebalbumPage(genfile.WebalbumFile):
             self.add_file_dependency(subtpl.path)
         return tpl
 
-    def _gen_other_img_link(self, img, dir=None, img_rel_path=None):
-        link_vals = {}
-        link_vals['link'] = img_rel_path
-        link_vals['link'] += self._add_size_qualifier(img.name + '.html',
-                                                      self.size_name)
-        link_vals['link'] = self.url_quote(link_vals['link'])
-        link_vals['thumb'] = img_rel_path
-        link_vals['thumb'] += self._add_size_qualifier(img.filename,
-                                                     genmedia.THUMB_SIZE_NAME)
-        link_vals['thumb'] = self.url_quote(link_vals['thumb'])
-
-        if not img.broken:
-            thumb = os.path.join(dir.path,
-                                 self._add_size_qualifier(img.filename,
-                                                     genmedia.THUMB_SIZE_NAME))
-            link_vals['thumb_width'],\
-                link_vals['thumb_height'] = img.get_size(thumb)
-
-            link_vals['thumb_name'] = self.dir.album._str_humanize(img.name)
-
-        return link_vals
-
-    def _gen_other_video_link(self, video, dir=None, vid_rel_path=None):
-        link_vals = {}
-        link_vals['link'] = vid_rel_path
-        link_vals['link'] += self._add_size_qualifier(video.name + '.html',
-                                                      self.size_name)
-        link_vals['link'] = self.url_quote(link_vals['link'])
-        link_vals['thumb'] = os.path.join(self.dir.source_dir.rel_root(),
-                                          'shared', 'videothumb.png')
-        link_vals['thumb'] = self.url_quote(link_vals['thumb'])
-
-        if not video.broken:
-            link_vals['thumb_width'],\
-                link_vals['thumb_height'] = video.get_size(True)
-
-            link_vals['thumb_name'] = self.dir.album._str_humanize(video.name)
-
-        return link_vals
+    def init_tpl_values(self):
+        tpl_values = {}
+        tpl_values.update(self.dir.tpl_vars)
+        return tpl_values
 
     def _gen_other_media_link(self, media, dir=None):
         if media:
-            if dir is None or dir is self.dir:
-                dir = self.dir
-                media_rel_path = ''
-            else:
-                media_rel_path = dir.source_dir.rel_path(self.dir.source_dir)\
-                                 + '/'
+            link_vals = {}
 
-            if media.type == 'image':
-                return self._gen_other_img_link(media, dir, media_rel_path)
-            elif media.type == 'video':
-                return self._gen_other_video_link(media, dir, media_rel_path)
-            else:
-                return None
+            link_vals['type'] = media.media.type
+
+            link_vals['link'] = media.browse_pages[self.size_name].rel_path(dir)
+            link_vals['link'] = self.url_quote(link_vals['link'])
+
+            link_vals['thumb'] = media.thumb.rel_path(dir)
+            link_vals['thumb'] = self.url_quote(link_vals['thumb'])
+
+            if not media.media.broken:
+                link_vals['thumb_width'],\
+                    link_vals['thumb_height'] = media.thumb.get_size()
+
+            link_vals['thumb_name'] = self.dir.album._str_humanize(media.media.name)
+
+            return link_vals
         else:
             return None
 
     def _get_osize_links(self, filename):
         osize_index_links = []
-        for osize_name in self.dir.album.browse_size_strings.keys():
+        for osize_name in self.dir.browse_sizes:
             osize_info = {}
             if osize_name == self.size_name:
                 # No link if we're on the current page
@@ -146,7 +117,7 @@ class WebalbumPage(genfile.WebalbumFile):
         if srcdir_path != self.dir.source_dir.path:
             index_path = srcdir_path
         if index_path is not None:
-            link_target = self.dir.rel_path(index_path) + link_target
+            link_target = self.dir.rel_path_to_src(index_path) + link_target
 
         return self.url_quote(link_target)
 
@@ -164,7 +135,7 @@ class WebalbumPage(genfile.WebalbumFile):
         return wg_path
 
     def _add_size_qualifier(self, path, size_name):
-        return self.dir.album._add_size_qualifier(path, size_name)
+        return self.dir._add_size_qualifier(path, size_name)
 
     def _do_not_escape(self, value):
         return genshi.core.Markup(value)
@@ -206,11 +177,11 @@ class WebalbumBrowsePage(WebalbumPage):
         self.load_tpl(self.media.type+'.thtml')
 
     def build(self):
-        page_rel_path = self._rel_path(self.dir.flattening_dir)
-        self.dir.album.log(_("  XHTML %s") % page_rel_path, 'info')
-        self.dir.album.log("(%s)" % self.page_path)
+        page_rel_path = self.rel_path(self.dir.flattening_dir)
+        logging.info(_("  XHTML %s") % page_rel_path)
+        logging.debug("(%s)" % self.page_path)
 
-        tpl_values = {}
+        tpl_values = self.init_tpl_values()
 
         # Breadcrumbs
         tpl_values['webgal_path'] = self._gen_webgal_path()
@@ -221,11 +192,11 @@ class WebalbumBrowsePage(WebalbumPage):
 
         prev = self.webalbum_media.previous
         if prev:
-            tpl_values['prev_link']  = self._gen_other_media_link(prev.media)
+            tpl_values['prev_link']  = self._gen_other_media_link(prev)
 
         next = self.webalbum_media.next
         if next:
-            tpl_values['next_link'] = self._gen_other_media_link(next.media)
+            tpl_values['next_link'] = self._gen_other_media_link(next)
 
         tpl_values['index_link'] = self._add_size_qualifier('index.html',
                                                             self.size_name)
@@ -236,11 +207,18 @@ class WebalbumBrowsePage(WebalbumPage):
         tpl_values['osize_links'] = self._get_osize_links(self.media.name)
         tpl_values['rel_root'] = self.dir.source_dir.rel_root()
 
-        if self.dir.album.original:
-            if self.dir.album.orig_base:
+        if self.dir.feed is not None:
+            tpl_values['feed_url'] = pathutils.relative_path(self.dir.path,
+                                                             self.dir.feed.path)
+            tpl_values['feed_url'] = self.url_quote(tpl_values['feed_url'])
+        else:
+            tpl_values['feed_url'] = None
+
+        if self.dir.original:
+            if self.dir.orig_base:
                 tpl_values['original_link'] = os.path.join(\
                     self.dir.source_dir.rel_root(),
-                    self.dir.album.orig_base,
+                    self.dir.orig_base,
                     self.dir.source_dir.strip_root(),
                     self.media.filename)
             else:
@@ -262,6 +240,8 @@ class WebalbumImagePage(WebalbumBrowsePage):
     def add_extra_vals(self, tpl_values):
         tpl_values['img_src'] = self._add_size_qualifier(self.image.filename,
                                                          self.size_name)
+        tpl_values['img_src'] = self.url_quote(tpl_values['img_src'])
+
         tpl_values['image_name'] = self.image.filename
 
         browse_image_path = os.path.join(self.dir.path,
@@ -302,7 +282,8 @@ class WebalbumVideoPage(WebalbumBrowsePage):
         self.video = self.media
 
     def add_extra_vals(self, tpl_values):
-        tpl_values['video_src'] = self.video.name + '.ogg'
+        tpl_values['video_src'] = self.video.name + '.webm'
+        tpl_values['video_src'] = self.url_quote(tpl_values['video_src'])
 
 
 class WebalbumIndexPage(WebalbumPage):
@@ -332,8 +313,8 @@ class WebalbumIndexPage(WebalbumPage):
                 # when dir is flattenend).
                 dir.add_dependency(media.browse_pages[size_name])
 
-            if self.dir.album.dirzip and dir.source_dir.get_media_count() > 1:
-                self.add_dependency(genfile.WebalbumArchive(dir))
+            if self.dir.dirzip is not None:
+                self.add_dependency(self.dir.dirzip)
 
         for subgal in self.subgals:
             self.add_dependency(subgal.source_dir)
@@ -387,8 +368,8 @@ class WebalbumIndexPage(WebalbumPage):
         if 'album_name' not in dir_info.keys():
             dir_info['album_name'] = dir.source_dir.human_name
 
-        if self.dir.album.dirzip and dir.dirzip:
-            archive_rel_path = dir.dirzip._rel_path(self.dir)
+        if dir.dirzip:
+            archive_rel_path = dir.dirzip.rel_path(self.dir)
             dir_info['dirzip'] = self.url_quote(archive_rel_path)
             dir_info['dirzip_size'] = self.format_filesize(dir.dirzip.size())
 
@@ -409,16 +390,16 @@ class WebalbumIndexPage(WebalbumPage):
                                          self._get_related_index_fn()])
             dir_info['link'] = self.url_quote(dir_info['link'])
             dir_info['album_picture'] = os.path.join(subgal.source_dir.name,
-                                     self.dir.album.get_webalbumpic_filename())
+                                            self.dir.get_webalbumpic_filename())
+            dir_info['album_picture'] = self.url_quote(dir_info['album_picture'])
             subgal_links.append(dir_info)
         return subgal_links
 
     def build(self):
-        self.dir.album.log(_("  XHTML %s") % os.path.basename(self.page_path),
-                           'info')
-        self.dir.album.log("(%s)" % self.page_path)
+        logging.info(_("  XHTML %s") % os.path.basename(self.page_path))
+        logging.debug("(%s)" % self.page_path)
 
-        values = {}
+        values = self.init_tpl_values()
 
         # Breadcrumbs (current is static, see dirindex.thtml, that's why the
         # last item of the list is removed).
@@ -439,9 +420,8 @@ class WebalbumIndexPage(WebalbumPage):
         values['medias'] = []
         for subdir, medias in self.galleries:
             info = self._get_dir_info(subdir)
-            media_links = map(lambda x: self._gen_other_media_link(x.media,
-                                                                   subdir),
-                              medias)
+            media_links = [self._gen_other_media_link(media, subdir)
+                           for media in medias]
             values['medias'].append((info, media_links, ))
 
         values.update(self._get_dir_info())
@@ -449,12 +429,22 @@ class WebalbumIndexPage(WebalbumPage):
         values['rel_root'] = self.dir.source_dir.rel_root()
         values['rel_path'] = self.dir.source_dir.strip_root()
 
+        if self.dir.feed is not None:
+            values['feed_url'] = pathutils.relative_path(self.dir.path,
+                                                         self.dir.feed.path)
+            values['feed_url'] = self.url_quote(values['feed_url'])
+        else:
+            values['feed_url'] = None
+
         self.page_template.dump(values, self.page_path)
 
 
 class WebalbumFeed(make.FileMakeObject):
 
-    def __init__(self, album, dest_dir, pub_url):
+    def __init__(self, album, dir_path, pub_url):
+        self.path = os.path.join(dir_path, 'index.xml')
+        super(WebalbumFeed, self).__init__(self.path)
+
         self.album = album
         self.pub_url = pub_url
         if not self.pub_url:
@@ -462,8 +452,6 @@ class WebalbumFeed(make.FileMakeObject):
         if not self.pub_url.endswith('/'):
             self.pub_url = self.pub_url + '/'
 
-        self.path = os.path.join(dest_dir, 'index.xml')
-        make.FileMakeObject.__init__(self, self.path)
         self.feed = feeds.RSS20(self.pub_url)
         self.item_template = self.album.tpl_loader.load('feeditem.thtml')
 
@@ -483,7 +471,7 @@ class WebalbumFeed(make.FileMakeObject):
 
         desc_values = {}
         desc_values['album_pic_path'] = os.path.join(url,
-                                          self.album.get_webalbumpic_filename())
+                                         webalbumdir.get_webalbumpic_filename())
         desc_values['subgal_count'] = webalbumdir.get_subgal_count()
         desc_values['picture_count'] = webalbumdir.source_dir.get_media_count('image')
         desc_values['desc'] = webalbumdir.source_dir.desc
@@ -493,8 +481,8 @@ class WebalbumFeed(make.FileMakeObject):
                             webalbumdir.source_dir.get_mtime())
 
     def build(self):
-        self.album.log(_("FEED %s") % os.path.basename(self.path), 'info')
-        self.album.log("(%s)" % self.path)
+        logging.info(_("FEED %s") % os.path.basename(self.path))
+        logging.debug("(%s)" % self.path)
         self.feed.dump(self.path)
 
 
@@ -515,9 +503,8 @@ class SharedFileTemplate(make.FileMakeObject):
         self.add_file_dependency(shared_tpl_name)
 
     def build(self):
-        self.album.log(_("TPL %%SHAREDDIR%%/%s")\
-                       % os.path.basename(self.path), 'info')
-        self.album.log("(%s)" % self.path)
+        logging.info(_("TPL %%SHAREDDIR%%/%s") % os.path.basename(self.path))
+        logging.debug("(%s)" % self.path)
         self.tpl.dump({}, self.path)
 
 
