@@ -120,7 +120,7 @@ class MakeTask(object):
             self.build()
         except KeyboardInterrupt:
             self.clean_output()
-            sys.exit(1)
+            raise
         self.stamp_build()
 
     def build(self):
@@ -208,7 +208,6 @@ class FileMakeObject(MakeTask):
             self.stamp_delete()
 
     def clean_output(self):
-        print "os.unlink(" + self._path + ")"
         os.unlink(self._path)
 
     def __repr__(self):
