@@ -21,7 +21,10 @@ import os
 import datetime
 import shutil
 
+import Image
+
 from __init__ import LazygalTestGen
+import lazygal.config
 from lazygal.generators import WebalbumDir
 from lazygal.sourcetree import Directory
 from lazygal import pyexiv2api as pyexiv2
@@ -91,10 +94,14 @@ class TestGenerators(LazygalTestGen):
                          [os.path.join(dest_path, 'extra_thumb.jpg')])
 
     def test_originals_symlinks(self):
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'original', 'Yes')
+        config.set('webgal', 'original-symlink', 'Yes')
+        self.setup_album(config)
+
         img_path = self.add_img(self.source_dir, 'symlink_target.jpg')
 
         dest_dir = self.get_working_path()
-        self.album.set_original(original=True, orig_symlink=True)
         self.album.generate(dest_dir)
 
         symlink = os.path.join(dest_dir, os.path.basename(img_path))
@@ -142,10 +149,35 @@ class TestGenerators(LazygalTestGen):
         def long(): return dest_image['Exif.GPSInfo.GPSLatitude'].value
         self.assertRaises(KeyError, long)
 
+    def test_resize_rotate_size(self):
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'image-size', 'std=800x600')
+        self.setup_album(config)
+
+        norotate_path = self.add_img(self.source_dir, 'norotate.jpg')
+        torotate_path = self.add_img(self.source_dir, 'torotate.jpg')
+        torotate = pyexiv2.ImageMetadata(torotate_path)
+        torotate.read()
+        torotate['Exif.Image.Orientation'] = 8
+        torotate.write()
+
+        # Generate album
+        dest_dir = self.get_working_path()
+        self.album.generate(dest_dir)
+
+        dest_norotate_path = os.path.join(dest_dir, 'norotate_std.jpg')
+        self.assertEqual(Image.open(dest_norotate_path).size, (800, 533, ))
+        dest_torotate_path = os.path.join(dest_dir, 'torotate_std.jpg')
+        self.assertEqual(Image.open(dest_torotate_path).size, (400, 600, ))
+
     def test_feed(self):
+        config = lazygal.config.LazygalConfig()
+        config.set('global', 'puburl', 'http://example.com/album/')
+        self.setup_album(config)
+
         img_path = self.add_img(self.source_dir, 'img01.jpg')
         dest_dir = self.get_working_path()
-        self.album.generate(dest_dir, 'http://example.com/album/')
+        self.album.generate(dest_dir)
 
         self.assertEqual(os.path.isfile(os.path.join(dest_dir, 'index.xml')),
                          True)
@@ -161,7 +193,9 @@ class TestSpecialGens(LazygalTestGen):
         '''
         It shall be possible to split big galleries on mutiple index pages.
         '''
-        self.setup_album({'thumbs_per_page': 4})
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'thumbs-per-page', 4)
+        self.setup_album(config)
 
         pics = [ 'img%d.jpg' % i for i in range(0, 9)]
         source_subgal = self.setup_subgal('subgal', pics)
@@ -171,7 +205,9 @@ class TestSpecialGens(LazygalTestGen):
         # for now...
 
     def test_flatten(self):
-        self.setup_album({'dir_flattening_depth': 1})
+        config = lazygal.config.LazygalConfig()
+        config.set('global', 'dir-flattening-depth', 1)
+        self.setup_album(config)
 
         source_subgal = self.setup_subgal('subgal', ['subgal_img.jpg'])
         self.album.generate(self.dest_path)
@@ -179,7 +215,10 @@ class TestSpecialGens(LazygalTestGen):
         # for now...
 
     def test_flattenpaginate(self):
-        self.setup_album({'thumbs_per_page': 4, 'dir_flattening_depth': 1,})
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'thumbs-per-page', 4)
+        config.set('global', 'dir-flattening-depth', 1)
+        self.setup_album(config)
 
         pics = [ 'img%d.jpg' % i for i in range(0, 9)]
         source_subgal = self.setup_subgal('subgal', pics)
@@ -239,6 +278,123 @@ class TestSpecialGens(LazygalTestGen):
         self.assertFalse(os.path.isdir(os.path.join(self.dest_path, 'symlinked', 'subgal', 'do_not_follow')))
 
 
+class TestSorting(LazygalTestGen):
+
+    def setUp(self):
+        super(TestSorting, self).setUp(False)
+        self.dest_path = os.path.join(self.tmpdir, 'dst')
+
+    def __setup_pics(self):
+        subgal_path = os.path.join(self.source_dir, 'subgal')
+        os.mkdir(subgal_path)
+
+        pics = ['4-december.jpg', '6-january.jpg', '1-february.jpg', '3-june.jpg', '5-august.jpg']
+        months = [12, 1, 2, 6, 8]
+        for index, pic in enumerate(pics):
+
+            img_path = self.add_img(subgal_path, pic)
+            img_exif = pyexiv2.ImageMetadata(img_path)
+            img_exif.read()
+            for tag in ('Exif.Photo.DateTimeDigitized',
+                        'Exif.Photo.DateTimeOriginal',
+                        'Exif.Image.DateTime',
+                       ):
+                img_exif[tag] = datetime.datetime(2011, months[index], 1)
+            img_exif.write()
+
+        return subgal_path, pics
+
+    def test_sortbyexif(self):
+        '''
+        It shall be possible to sort images in a gallery according to EXIF date.
+        '''
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'sort-medias', 'exif')
+        self.setup_album(config)
+        subgal_path, pics = self.__setup_pics()
+
+        src_dir = Directory(subgal_path, [], pics, self.album)
+        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)
+
+        dest_subgal.sort_task.make()
+
+        self.assertEqual([media.media.filename for media in dest_subgal.medias],
+                         [u'6-january.jpg', u'1-february.jpg', u'3-june.jpg', u'5-august.jpg', u'4-december.jpg'])
+
+    def test_sortbyexif_galsplit(self):
+        '''
+        It shall be possible to sort images in galleries split on multiple
+        pages according to EXIF date.
+        '''
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'sort-medias', 'exif')
+        config.set('webgal', 'thumbs-per-page', 3)
+        self.setup_album(config)
+        subgal_path, pics = self.__setup_pics()
+
+        src_dir = Directory(subgal_path, [], pics, self.album)
+        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)
+
+        dest_subgal.sort_task.make()
+
+        self.assertEqual([media.media.filename for media in dest_subgal.medias],
+                         [u'6-january.jpg', u'1-february.jpg', u'3-june.jpg', u'5-august.jpg', u'4-december.jpg'])
+
+        # page #1
+        page_medias = dest_subgal.index_pages[0][0].galleries[0][1]
+        self.assertEqual([media.media.filename for media in page_medias],
+                         [u'6-january.jpg', u'1-february.jpg', u'3-june.jpg'])
+        # page #2
+        page_medias = dest_subgal.index_pages[1][0].galleries[0][1]
+        self.assertEqual([media.media.filename for media in page_medias],
+                         [u'5-august.jpg', u'4-december.jpg'])
+
+    def test_sortbyfilename(self):
+        '''
+        It shall be possible to sort images in a gallery by filename.
+        '''
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'sort-medias', 'filename')
+        self.setup_album(config)
+        subgal_path, pics = self.__setup_pics()
+
+        src_dir = Directory(subgal_path, [], pics, self.album)
+        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)
+
+        dest_subgal.sort_task.make()
+
+        self.assertEqual([media.media.filename for media in dest_subgal.medias],
+                         [u'1-february.jpg', u'3-june.jpg', u'4-december.jpg', u'5-august.jpg', u'6-january.jpg'])
+
+    def test_sortbyfilename_galsplit(self):
+        '''
+        It shall be possible to sort images in galleries split on multiple
+        pages by filename.
+        '''
+        config = lazygal.config.LazygalConfig()
+        config.set('webgal', 'sort-medias', 'filename')
+        config.set('webgal', 'thumbs-per-page', 3)
+        self.setup_album(config)
+        subgal_path, pics = self.__setup_pics()
+
+        src_dir = Directory(subgal_path, [], pics, self.album)
+        dest_subgal = WebalbumDir(src_dir, [], self.album, self.dest_path)
+
+        dest_subgal.sort_task.make()
+
+        self.assertEqual([media.media.filename for media in dest_subgal.medias],
+                         [u'1-february.jpg', u'3-june.jpg', u'4-december.jpg', u'5-august.jpg', u'6-january.jpg'])
+
+        # page #1
+        page_medias = dest_subgal.index_pages[0][0].galleries[0][1]
+        self.assertEqual([media.media.filename for media in page_medias],
+                         [u'1-february.jpg', u'3-june.jpg', u'4-december.jpg'])
+        # page #2
+        page_medias = dest_subgal.index_pages[1][0].galleries[0][1]
+        self.assertEqual([media.media.filename for media in page_medias],
+                         [u'5-august.jpg', u'6-january.jpg'])
+
+
 if __name__ == '__main__':
     unittest.main()
 
